#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class HouseholdLanguageDownloader(BaseTableConfig):
    YEAR_LIST = (2017, 2016)
    PROCESSED_TABLE_NAME = 'householdlanguage'
    UNIVERSE = "households"
    RAW_TABLE_NAME = 'C16002'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "universe",
        "002": "only_english",
        "003": "total_spanish",
        "004": "spanish_limited_english",
        "005": "spanish_not_limited_english",
        "006": "total_other_indo_european",
        "007": "other_indo_european_limited_english",
        "008": "other_indo_european_not_limited_english",
        "009": "total_asian_and_pacific_islander",
        "010": "asian_and_pacific_islander_limited_english",
        "011": "asian_and_pacific_islander_not_limited_english",
        "012": "total_other",
        "013": "other_limited_english",
        "014": "other_not_limited_english",
    })

    def process(self, *args, **kwargs):
        """
        Combine language counts to get total english/non-english speakers
        """
        df = super().process(*args, **kwargs)

        languages = [
            'spanish',
            'other_indo_european',
            'asian_and_pacific_islander',
            'other'
        ]

        # English vs. no English totals
        df['total_not_limited_english'] = df['only_english'] + df[
            [f'{l}_not_limited_english' for l in languages]
        ].sum(axis=1)
        df['total_limited_english'] = df[
            [f'{l}_limited_english' for l in languages]
        ].sum(axis=1)

        # Pass it out
        return df


@register
class LanguageShortFormDownloader(BaseTableConfig):
    YEAR_LIST = (2017, 2016)
    PROCESSED_TABLE_NAME = 'languageshortform'
    UNIVERSE = "population 5 years and over"
    RAW_TABLE_NAME = 'C16001'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': 'universe',
        '002': 'only_english',
        '003': 'total_spanish',
        '004': 'spanish_and_english_very_well',
        '005': 'spanish_and_english_less_than_very_well',
        '006': 'total_french_haitan_or_cajun',
        '007': 'french_haitan_or_cajun_and_english_very_well',
        '008': 'french_haitan_or_cajun_and_english_less_than_very_well',
        '009': 'total_german_or_other_west_germanic',
        '010': 'german_or_other_west_germanic_and_english_very_well',
        '011': 'german_or_other_west_germanic_and_english_less_than_very_well',
        '012': 'total_russian_polish_or_other_slavic',
        '013': 'russian_polish_or_other_slavic_and_english_very_well',
        '014': 'russian_polish_or_other_slavic_and_english_less_than_very_well',
        '015': 'total_other_indo_european',
        '016': 'other_indo_european_and_english_very_well',
        '017': 'other_indo_european_and_english_less_than_very_well',
        '018': 'total_korean',
        '019': 'korean_and_english_very_well',
        '020': 'korean_and_english_less_than_very_well',
        '021': 'total_chinese',
        '022': 'chinese_and_english_very_well',
        '023': 'chinese_and_english_less_than_very_well',
        '024': 'total_vietnamese',
        '025': 'vietnamese_and_english_very_well',
        '026': 'vietnamese_and_english_less_than_very_well',
        '027': 'total_tagalog',
        '028': 'tagalog_and_english_very_well',
        '029': 'tagalog_and_english_less_than_very_well',
        '030': 'total_other_asian',
        '031': 'other_asian_and_english_very_well',
        '032': 'other_asian_and_english_less_than_very_well',
        '033': 'total_arabic',
        '034': 'arabic_and_english_very_well',
        '035': 'arabic_and_english_less_than_very_well',
        '036': 'total_other',
        '037': 'other_and_english_very_well',
        '038': 'other_and_english_less_than_very_well'
    })

    def process(self, *args, **kwargs):
        """
        Combine language counts to get total english/non-english speakers
        """
        df = super().process(*args, **kwargs)

        languages = [
            'spanish',
            'french_haitan_or_cajun',
            'german_or_other_west_germanic',
            'russian_polish_or_other_slavic',
            'other_indo_european',
            'korean',
            'chinese',
            'vietnamese',
            'tagalog',
            'other_asian',
            'arabic',
            'other'
        ]

        # English vs. no English totals
        df['total_english'] = df['only_english'] + df[
            [f'{l}_and_english_very_well' for l in languages]
        ].sum(axis=1)
        df['total_english_less_than_very_well'] = df[
            [f'{l}_and_english_less_than_very_well' for l in languages]
        ].sum(axis=1)

        # Pass it out
        return df


@register
class LanguageLongFormDownloader(BaseTableConfig):
    YEAR_LIST = (2017, 2016)
    GEOTYPE_LIST = ("nationwide", "states", "counties", "congressional_districts", "places")
    PROCESSED_TABLE_NAME = 'languagelongform'
    UNIVERSE = "population 5 years and over"
    RAW_TABLE_NAME = 'B16001'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': 'universe',
        '002': 'only_english',
        '003': 'total_spanish',
        '004': 'spanish_and_english_very_well',
        '005': 'spanish_and_english_less_than_very_well',
        '006': 'total_french',
        '007': 'french_and_english_very_well',
        '008': 'french_and_english_less_than_very_well',
        '009': 'total_haitian',
        '010': 'haitian_and_english_very_well',
        '011': 'haitian_and_english_less_than_very_well',
        '012': 'total_italian',
        '013': 'italian_and_english_very_well',
        '014': 'italian_and_english_less_than_very_well',
        '015': 'total_portuguese',
        '016': 'portuguese_and_english_very_well',
        '017': 'portuguese_and_english_less_than_very_well',
        '018': 'total_german',
        '019': 'german_and_english_very_well',
        '020': 'german_and_english_less_than_very_well',
        '021': 'total_other_germanic',
        '022': 'other_germanic_and_english_very_well',
        '023': 'other_germanic_and_english_less_than_very_well',
        '024': 'total_greek',
        '025': 'greek_and_english_very_well',
        '026': 'greek_and_english_less_than_very_well',
        '027': 'total_russian',
        '028': 'russian_and_english_very_well',
        '029': 'russian_and_english_less_than_very_well',
        '030': 'total_polish',
        '031': 'polish_and_english_very_well',
        '032': 'polish_and_english_less_than_very_well',
        '033': 'total_serbo_croatian',
        '034': 'serbo_croatian_and_english_very_well',
        '035': 'serbo_croatian_and_english_less_than_very_well',
        '036': 'total_other_slavic',
        '037': 'other_slavic_and_english_very_well',
        '038': 'other_slavic_and_english_less_than_very_well',
        '039': 'total_armenian',
        '040': 'armenian_and_english_very_well',
        '041': 'armenian_and_english_less_than_very_well',
        '042': 'total_persian',
        '043': 'persian_and_english_very_well',
        '044': 'persian_and_english_less_than_very_well',
        '045': 'total_gujarati',
        '046': 'gujarati_and_english_very_well',
        '047': 'gujarati_and_english_less_than_very_well',
        '048': 'total_hindi',
        '049': 'hindi_and_english_very_well',
        '050': 'hindi_and_english_less_than_very_well',
        '051': 'total_urdu',
        '052': 'urdu_and_english_very_well',
        '053': 'urdu_and_english_less_than_very_well',
        '054': 'total_punjabi',
        '055': 'punjabi_and_english_very_well',
        '056': 'punjabi_and_english_less_than_very_well',
        '057': 'total_bengali',
        '058': 'bengali_and_english_very_well',
        '059': 'bengali_and_english_less_than_very_well',
        '060': 'total_other_indic',
        '061': 'other_indic_and_english_very_well',
        '062': 'other_indic_and_english_less_than_very_well',
        '063': 'total_other_indoeuropean',
        '064': 'other_indoeuropean_and_english_very_well',
        '065': 'other_indoeuropean_and_english_less_than_very_well',
        '066': 'total_telugu',
        '067': 'telugu_and_english_very_well',
        '068': 'telugu_and_english_less_than_very_well',
        '069': 'total_tamil',
        '070': 'tamil_and_english_very_well',
        '071': 'tamil_and_english_less_than_very_well',
        '072': 'total_other_dravidian',
        '073': 'other_dravidian_and_english_very_well',
        '074': 'other_dravidian_and_english_less_than_very_well',
        '075': 'total_chinese',
        '076': 'chinese_and_english_very_well',
        '077': 'chinese_and_english_less_than_very_well',
        '078': 'total_japanese',
        '079': 'japanese_and_english_very_well',
        '080': 'japanese_and_english_less_than_very_well',
        '081': 'total_korean',
        '082': 'korean_and_english_very_well',
        '083': 'korean_and_english_less_than_very_well',
        '084': 'total_hmong',
        '085': 'hmong_and_english_very_well',
        '086': 'hmong_and_english_less_than_very_well',
        '087': 'total_vietnamese',
        '088': 'vietnamese_and_english_very_well',
        '089': 'vietnamese_and_english_less_than_very_well',
        '090': 'total_khmer',
        '091': 'khmer_and_english_very_well',
        '092': 'khmer_and_english_less_than_very_well',
        '093': 'total_other_tai_kadai',
        '094': 'other_tai_kadai_and_english_very_well',
        '095': 'other_tai_kadai_and_english_less_than_very_well',
        '096': 'total_other_asia',
        '097': 'other_asia_and_english_very_well',
        '098': 'other_asia_and_english_less_than_very_well',
        '099': 'total_tagalog',
        '100': 'tagalog_and_english_very_well',
        '101': 'tagalog_and_english_less_than_very_well',
        '102': 'total_other_austronesian',
        '103': 'other_austronesian_and_english_very_well',
        '104': 'other_austronesian_and_english_less_than_very_well',
        '105': 'total_arabic',
        '106': 'arabic_and_english_very_well',
        '107': 'arabic_and_english_less_than_very_well',
        '108': 'total_hebrew',
        '109': 'hebrew_and_english_very_well',
        '110': 'hebrew_and_english_less_than_very_well',
        '111': 'total_other_afroasiatic',
        '112': 'other_afroasiatic_and_english_very_well',
        '113': 'other_afroasiatic_and_english_less_than_very_well',
        '114': 'total_other_western_africa',
        '115': 'other_western_africa_and_english_very_well',
        '116': 'other_western_africa_and_english_less_than_very_well',
        '117': 'total_other_central_eastern_southern_africa',
        '118': 'other_central_eastern_southern_africa_and_english_very_well',
        '119': 'other_central_eastern_southern_africa_and_english_less_than_very_well',
        '120': 'total_navajo',
        '121': 'navajo_and_english_very_well',
        '122': 'navajo_and_english_less_than_very_well',
        '123': 'total_other_native_north_america',
        '124': 'other_native_north_america_and_english_very_well',
        '125': 'other_native_north_america_and_english_less_than_very_well',
        '126': 'total_other_unspecified',
        '127': 'other_unspecified_and_english_very_well',
        '128': 'other_unspecified_and_english_less_than_very_well'
    })

    def process(self, *args, **kwargs):
        """
        Combine language counts to get total english/non-english speakers
        """
        df = super().process(*args, **kwargs)

        languages = [
            'spanish',
            'french',
            'haitian',
            'italian',
            'portuguese',
            'german',
            'other_germanic',
            'greek',
            'russian',
            'polish',
            'serbo_croatian',
            'other_slavic',
            'armenian',
            'persian',
            'gujarati',
            'hindi',
            'urdu',
            'punjabi',
            'bengali',
            'other_indic',
            'other_indoeuropean',
            'telugu',
            'tamil',
            'other_dravidian',
            'chinese',
            'japanese',
            'korean',
            'hmong',
            'vietnamese',
            'khmer',
            'other_tai_kadai',
            'other_asia',
            'tagalog',
            'other_austronesian',
            'arabic',
            'hebrew',
            'other_afroasiatic',
            'other_western_africa',
            'other_central_eastern_southern_africa',
            'navajo',
            'other_native_north_america',
            'other_unspecified'
        ]

        # English vs. no English totals
        df['total_english'] = df['only_english'] + df[[f'{l}_and_english_very_well' for l in languages]].sum(axis=1)
        df['total_english_less_than_very_well'] = df[
            [f'{l}_and_english_less_than_very_well' for l in languages]
        ].sum(axis=1)

        # Group into the four language groups defined by the Census
        # https://www.census.gov/topics/population/language-use/about.html
        # Calculate our custom groups (other than Spanish, which we already have)
        groupsets = collections.OrderedDict({
            "other_indo_european_group": [
                'french',
                'haitian',
                'italian',
                'portuguese',
                'german',
                'other_germanic',
                'greek',
                'russian',
                'polish',
                'serbo_croatian',
                'other_slavic',
                'armenian',
                'persian',
                'gujarati',
                'hindi',
                'urdu',
                'punjabi',
                'bengali',
                'other_indic',
                'other_indoeuropean',
                'telugu',
                'tamil',
                'other_dravidian'
            ],
            "asian_and_pacific_island_group": [
                'chinese',
                'japanese',
                'korean',
                'hmong',
                'vietnamese',
                'khmer',
                'other_tai_kadai',
                'other_asia',
                'tagalog',
                'other_austronesian'
            ],
            "all_other_group": [
                'arabic',
                'hebrew',
                'other_afroasiatic',
                'other_western_africa',
                'other_central_eastern_southern_africa',
                'navajo',
                'other_native_north_america',
                'other_unspecified'
            ]
        })
        for groupset, group_list in groupsets.items():
            df[f'total_{groupset}'] = df[
                [f'total_{f}' for f in group_list]
            ].sum(axis=1)
            df[f'{groupset}_and_english_very_well'] = df[
                [f'{f}_and_english_very_well' for f in group_list]
            ].sum(axis=1)
            df[f'{groupset}_and_english_less_than_very_well'] = df[
                [f'{f}_and_english_less_than_very_well' for f in group_list]
            ].sum(axis=1)

        # Pass it back
        return df



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class HouseholdIncomeDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "householdincome"
    UNIVERSE = "households"
    RAW_TABLE_NAME = 'B19001'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "0_9999",
        '003': "10000_15000",
        '004': "15000_20000",
        '005': "20000_25000",
        '006': "25000_30000",
        '007': "30000_35000",
        '008': "35000_40000",
        '009': "40000_45000",
        '010': "45000_50000",
        '011': "50000_60000",
        '012': "60000_75000",
        '013': "75000_100000",
        '014': "100000_125000",
        '015': "125000_150000",
        '016': "150000_200000",
        '017': "200000_more"
    })


@register
class HouseholdIncomeLatinoDownloader(HouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "householdincomelatino"
    RAW_TABLE_NAME = 'B19001I'


@register
class HouseholdIncomeWhiteDownloader(HouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "householdincomewhite"
    RAW_TABLE_NAME = 'B19001H'


@register
class HouseholdIncomeBlackDownloader(HouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "householdincomeblack"
    RAW_TABLE_NAME = 'B19001B'


@register
class HouseholdIncomeAsianDownloader(HouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "householdincomeasian"
    RAW_TABLE_NAME = 'B19001D'



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class MedianHouseholdIncomeDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "medianhouseholdincome"
    UNIVERSE = "households"
    RAW_TABLE_NAME = 'B19013'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "median"
    })


@register
class MedianHouseholdIncomeLatinoDownloader(MedianHouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "medianhouseholdincomelatino"
    RAW_TABLE_NAME = 'B19013I'


@register
class MedianHouseholdIncomeWhiteDownloader(MedianHouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "medianhouseholdincomewhite"
    RAW_TABLE_NAME = 'B19013H'


@register
class MedianHouseholdIncomeBlackDownloader(MedianHouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "medianhouseholdincomeblack"
    RAW_TABLE_NAME = 'B19013B'


@register
class MedianHouseholdIncomeAsianDownloader(MedianHouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "medianhouseholdincomeasian"
    RAW_TABLE_NAME = 'B19013D'



import collections

# https://www.census.gov/data/developers/data-sets/acs-1year/notes-on-acs-estimate-and-annotation-values.html

ESTIMATE_MAP = collections.OrderedDict({
    "-999999999": "too few samples",
    "-888888888": "not applicable",
    "-666666666": "estimate can't be calculated",
    "*": "significantly different from most current year",
    "C": "controlled so don't use tests",
    # "+": "",
    # "-": "falls in the lowest interval"
})
MOE_MAP = collections.OrderedDict({
    "N": "too few samples",
    "(X)": "not applicable",
    "-": "too few samples or ratio of medians cannot be calculated",
    "*****": "estimate is controlled",
    "***": "falls in lowest interval or highest interval",
    "**": "too few samples to calculate standard error",
    "-555555555": "estimate is controlled",
    "-333333333": "falls in lowest interval or highest interval",
    "-222222222": "too few samples to calculate standard error",
    "N/A": "significantly different from most current year"
})



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class HousingValueDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "housingvalue"
    UNIVERSE = "owner occupied housing units"
    RAW_TABLE_NAME = "B25075"
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "01": "total",
        "02": "0_10000",
        "03": "10000_14999",
        "04": "15000_19999",
        "05": "20000_24999",
        "06": "25000_29999",
        "07": "30000_34999",
        "08": "35000_39999",
        "09": "40000_49999",
        "10": "50000_59999",
        "11": "60000_69999",
        "12": "70000_79999",
        "13": "80000_89999",
        "14": "90000_99999",
        "15": "100000_124999",
        "16": "125000_149999",
        "17": "150000_174999",
        "18": "175000_199999",
        "19": "200000_249999",
        "20": "250000_299999",
        "21": "300000_399999",
        "22": "400000_499999",
        "23": "500000_749999",
        "24": "750000_999999",
        "25": "1000000_1499999",
        "26": "1500000_1999999",
        "27": "2000000_more",
    })


@register
class MedianMonthlyHousingCostsDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "medianmonthlyhousingcosts"
    UNIVERSE = "occupied housing units with monthly housing costs"
    RAW_TABLE_NAME = "B25105"
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "median"
    })


@register
class MedianHousingValueDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "medianhousingvalue"
    UNIVERSE = "owner-occupied housing units"
    RAW_TABLE_NAME = 'B25077'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "median"
    })


@register
class MedianGrossRentDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "mediangrossrent"
    UNIVERSE = "renter-occupied housing units paying cash rent"
    RAW_TABLE_NAME = 'B25064'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "median"
    })


@register
class TenureDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "tenure"
    UNIVERSE = "occupied housing units"
    RAW_TABLE_NAME = 'B25003'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': 'universe',
        '002': 'owner_occupied',
        '003': 'renter_occupied'
    })


@register
class TenureLatinoDownloader(TenureDownloader):
    PROCESSED_TABLE_NAME = "tenurelatino"
    RAW_TABLE_NAME = 'B25003I'


@register
class TenureWhiteDownloader(TenureDownloader):
    PROCESSED_TABLE_NAME = "tenurewhite"
    RAW_TABLE_NAME = 'B25003H'


@register
class TenureBlackDownloader(TenureDownloader):
    PROCESSED_TABLE_NAME = "tenureblack"
    RAW_TABLE_NAME = 'B25003B'


@register
class TenureAsianDownloader(TenureDownloader):
    PROCESSED_TABLE_NAME = "tenureasian"
    RAW_TABLE_NAME = 'B25003D'



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class EducationDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = 'education'
    UNIVERSE = "population 25 Years and Over"
    RAW_TABLE_NAME = 'B15002'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "male_total",
        '003': "male_no_schooling",
        '004': "male_nursery_to_4th_grade",
        '005': "male_5th_and_6th_grade",
        '006': "male_7th_and_8th_grade",
        '007': "male_9th_grade",
        '008': "male_10th_grade",
        '009': "male_11th_grade",
        '010': "male_12th_grade_no_diploma",
        '011': "male_high_school_graduate",
        '012': "male_less_than_1_year_college",
        '013': "male_1_or_more_years_college",
        '014': "male_associates_degree",
        '015': "male_bachelors_degree",
        '016': "male_masters_degree",
        '017': "male_professional_school_degree",
        '018': "male_doctorate_degree",
        '019': "female_total",
        '020': "female_no_schooling",
        '021': "female_nursery_to_4th_grade",
        '022': "female_5th_and_6th_grade",
        '023': "female_7th_and_8th_grade",
        '024': "female_9th_grade",
        '025': "female_10th_grade",
        '026': "female_11th_grade",
        '027': "female_12th_grade_no_diploma",
        '028': "female_high_school_graduate",
        '029': "female_less_than_1_year_college",
        '030': "female_1_or_more_years_college",
        '031': "female_associates_degree",
        '032': "female_bachelors_degree",
        '033': "female_masters_degree",
        '034': "female_professional_school_degree",
        '035': "female_doctorate_degree",
    })

    def process(self, *args, **kwargs):
        """
        Combine raw age columns into our preferred groupings.
        """
        df = super().process(*args, **kwargs)

        # Calculate totals for both genders together
        groups = [
            "no_schooling",
            "nursery_to_4th_grade",
            "5th_and_6th_grade",
            "7th_and_8th_grade",
            "9th_grade",
            "10th_grade",
            "11th_grade",
            "12th_grade_no_diploma",
            "high_school_graduate",
            "less_than_1_year_college",
            "1_or_more_years_college",
            "associates_degree",
            "bachelors_degree",
            "masters_degree",
            "professional_school_degree",
            "doctorate_degree",
        ]
        for g in groups:
            df[f'total_{g}'] = df[f'male_{g}'] + df[f'female_{g}']

        # Calculate our custom groups
        groupsets = collections.OrderedDict({
            "less_than_high_scool": [
                "no_schooling",
                "nursery_to_4th_grade",
                "5th_and_6th_grade",
                "7th_and_8th_grade",
                "9th_grade",
                "10th_grade",
                "11th_grade",
                "12th_grade_no_diploma"
            ],
            "some_college": [
                "less_than_1_year_college",
                "1_or_more_years_college"
            ],
            "masters_or_higher": [
                "masters_degree",
                "professional_school_degree",
                "doctorate_degree"
            ],
            "bachelors_or_higher": [
                "bachelors_degree",
                "masters_degree",
                "professional_school_degree",
                "doctorate_degree"
            ]
        })
        for groupset, group_list in groupsets.items():
            df[f'total_{groupset}'] = df[[f'total_{f}' for f in group_list]].sum(axis=1)
            df[f'male_{groupset}'] = df[[f'male_{f}' for f in group_list]].sum(axis=1)
            df[f'female_{groupset}'] = df[[f'female_{f}' for f in group_list]].sum(axis=1)

        # Pass it back
        return df



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class AncestryDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "ancestry"
    UNIVERSE = "people reporting single ancestry"
    RAW_TABLE_NAME = 'B04004'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "afghan",
        '003': "albanian",
        '004': "alsatian",
        '005': "american",
        '006': "arab",
        '007': "egyptian",
        '008': "iraqi",
        '009': "jordanian",
        '010': "lebanese",
        '011': "moroccan",
        '012': "palestinian",
        '013': "syrian",
        '014': "all_arab",
        '015': "other_arab",
        '016': "armenian",
        '017': "assyrian_chaldean_syriac",
        '018': "australian",
        '019': "austrian",
        '020': "basque",
        '021': "belgian",
        '022': "brazilian",
        '023': "british",
        '024': "bulgarian",
        '025': "cajun",
        '026': "canadian",
        '027': "carpatho_rusyn",
        '028': "celtic",
        '029': "croatian",
        '030': "cypriot",
        '031': "czech",
        '032': "czechoslovakian",
        '033': "danish",
        '034': "dutch",
        '035': "eastern_european",
        '036': "english",
        '037': "estonian",
        '038': "european",
        '039': "finnish",
        '040': "french_except_basque",
        '041': "french_canadian",
        '042': "german",
        '043': "german_russian",
        '044': "greek",
        '045': "guyanese",
        '046': "hungarian",
        '047': "icelander",
        '048': "iranian",
        '049': "irish",
        '050': "israeli",
        '051': "italian",
        '052': "latvian",
        '053': "lithuanian",
        '054': "luxemburger",
        '055': "macedonian",
        '056': "maltese",
        '057': "new_zealander",
        '058': "northern_european",
        '059': "norwegian",
        '060': "pennsylvania_german",
        '061': "polish",
        '062': "portuguese",
        '063': "romanian",
        '064': "russian",
        '065': "scandinavian",
        '066': "scotch-irish",
        '067': "scottish",
        '068': "serbian",
        '069': "slavic",
        '070': "slovak",
        '071': "slovene",
        '072': "soviet_union",
        '073': "subsaharan_african",
        '074': "cape_verdean",
        '075': "ethiopian",
        '076': "ghanaian",
        '077': "kenyan",
        '078': "liberian",
        '079': "nigerian",
        '080': "senegalese",
        '081': "sierra_leonean",
        '082': "somali",
        '083': "south_african",
        '084': "sudanese",
        '085': "ugandan",
        '086': "zimbabwean",
        '087': "african",
        '088': "other_subsaharan_african",
        '089': "swedish",
        '090': "swiss",
        '091': "turkish",
        '092': "ukranian",
        '093': "welsh",
        '094': "west_indian_except_hispanic_groups",
        '095': "bahamian",
        '096': "barbadian",
        '097': "belizean",
        '098': "bermudan",
        '099': "british_west_indian",
        '100': "dutch_west_indian",
        '101': "haitian",
        '102': "jamaican",
        '103': "trinidadian_and_tobagonian",
        '104': "us_virgin_islander",
        '105': "west_indian",
        '106': "other_west_indian",
        '107': "yugoslavian",
        '108': "other_groups"
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class MedianAgeDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = 'medianage'
    UNIVERSE = "total population"
    RAW_TABLE_NAME = 'B01002'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "median",
        "002": "male",
        "003": "female"
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class MobilityDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "mobility"
    UNIVERSE = "population 1 year and over"
    RAW_TABLE_NAME = 'B07003'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '004': "same_house",
        '007': "moved_within_county",
        '010': "moved_from_different_county_in_same_state",
        '013': "moved_from_different_state",
        '016': "moved_from_abroad",
    })


@register
class MobilityBySexDownloader(MobilityDownloader):
    PROCESSED_TABLE_NAME = "mobilitybysex"
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "male_total",
        '003': "female_total",
        '004': "total_same_house",
        '005': "male_same_house",
        '006': "female_same_house",
        '007': "total_moved_within_county",
        '008': "male_moved_within_county",
        '009': "female_moved_within_county",
        '010': "total_moved_from_different_county_in_same_state",
        '011': "male_moved_from_different_county_in_same_state",
        '012': "female_moved_from_different_county_in_same_state",
        '013': "total_moved_from_different_state",
        '014': "male_moved_from_different_state",
        '015': "female_moved_from_different_state",
        '016': "total_moved_from_abroad",
        '017': "male_moved_from_abroad",
        '018': "female_moved_from_abroad"
    })


@register
class MobilityWhiteDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "mobilitywhite"
    UNIVERSE = "population 1 year and over"
    RAW_TABLE_NAME = 'B07004H'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "same_house",
        '003': "moved_within_county",
        '004': "moved_from_different_county_in_same_state",
        '005': "moved_from_different_state",
        '006': "moved_from_abroad"
    })


@register
class MobilityBlackDownloader(MobilityWhiteDownloader):
    PROCESSED_TABLE_NAME = "mobilityblack"
    RAW_TABLE_NAME = 'B07004B'


@register
class MobilityAsianDownloader(MobilityWhiteDownloader):
    PROCESSED_TABLE_NAME = "mobilityasian"
    RAW_TABLE_NAME = 'B07004D'


@register
class MobilityLatinoDownloader(MobilityWhiteDownloader):
    PROCESSED_TABLE_NAME = "mobilitylatino"
    RAW_TABLE_NAME = 'B07004I'


@register
class MobilityByCitizenship(BaseTableConfig):
    PROCESSED_TABLE_NAME = "mobilitybycitizenship"
    UNIVERSE = "population 1 year and over"
    RAW_TABLE_NAME = 'B07007'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "native",
        '003': "foreign_born",
        '004': "foreign_born_naturalized_us_citizen",
        '005': "foreign_born_not_us_citizen",
        '006': "same_house_1_year_ago",
        '007': "same_house_1_year_ago_native",
        '008': "same_house_1_year_ago_foreign_born",
        '009': "same_house_1_year_ago_foreign_born_naturalized_us_citizen",
        '010': "same_house_1_year_ago_foreign_born_not_us_citizen",
        '011': "moved_within_same_county",
        '012': "moved_within_same_county_native",
        '013': "moved_within_same_county_foreign_born",
        '014': "moved_within_same_county_foreign_born_naturalized_us_citizen",
        '015': "moved_within_same_county_foreign_born_not_us_citizen",
        '016': "moved_from_different_county_within_same_state",
        '017': "moved_from_different_county_within_same_state_native",
        '018': "moved_from_different_county_within_same_state_foreign_born",
        '019': "moved_from_different_county_within_same_state_foreign_born_naturalized_us_citizen",
        '020': "moved_from_different_county_within_same_state_foreign_born_not_us_citizen",
        '021': "moved_from_different_state",
        '022': "moved_from_different_state_native",
        '023': "moved_from_different_state_foreign_born",
        '024': "moved_from_different_state_foreign_born_naturalized_us_citizen",
        '025': "moved_from_different_state_foreign_born_not_us_citizen",
        '026': "moved_from_abroad",
        '027': "moved_from_abroad_native",
        '028': "moved_from_abroad_foreign_born",
        '029': "moved_from_abroad_foreign_born_naturalized_us_citizen",
        '030': "moved_from_abroad_foreign_born_not_us_citizen"
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class EmploymentStatusDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "employmentstatus"
    UNIVERSE = "population 16 years and over"
    RAW_TABLE_NAME = 'B23025'
    YEAR_LIST = (
        2017,
        2016,
        2015,
        2014,
        2013,
        2012,
        2011
    )
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': 'universe',
        '002': 'in_labor_force',
        '003': 'civilian',
        '004': 'civilian_employed',
        '005': 'civilian_unemployed',
        '006': 'armed_forces',
        '007': 'not_in_labor_force'
    })
